import { Host, h } from '@stencil/core';
export class WebAccordionItem {
  constructor() {
    this.calculatedHeight = 0;
    this.transitioning = false;
    this.index = -1;
    this.open = false;
    this.mutationObserverConfig = { childList: true, subtree: true };
  }
  get style() {
    return {
      height: this.open ? this.calculatedHeight + "px" : "0px"
    };
  }
  stateChanged() {
    this.transitioning = true;
  }
  componentWillLoad() {
    const children = this.element.parentElement.querySelectorAll('web-accordion-item');
    for (let i = 0; i < children.length; i++) {
      const child = children[i];
      if (child == this.element) {
        this.index = i;
      }
    }
    this.mutationObserver = new MutationObserver(() => this.contentChanged.emit());
    this.mutationObserver.observe(this.element, this.mutationObserverConfig);
  }
  componentDidLoad() {
    this.calculateHeight();
  }
  disconnectedCallback() {
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
    }
  }
  /**
   * recalculate Height of accordion item (e.g., when the content of the item changes)
   */
  recalculateHeight() {
    const oldCalculatedHeight = this.calculatedHeight;
    if (this.calculateHeight() != oldCalculatedHeight && this.open) {
      this.transitioning = true;
    }
  }
  /**
   * close the accordion item
   */
  async closeItem() {
    this.open = false;
  }
  /**
   * open the accordion item
   */
  async openItem() {
    this.open = true;
    this.openEvent.emit({
      index: this.index
    });
  }
  toggle() {
    if (this.open) {
      this.closeItem();
    }
    else {
      this.openItem();
    }
  }
  handleTransitionEnd() {
    this.transitioning = false;
  }
  calculateHeight() {
    this.calculatedHeight = this.element.querySelector('web-accordion-item > :not([slot="title"])').clientHeight;
    return this.calculatedHeight;
  }
  render() {
    return (h(Host, null, h("header", { onClick: () => this.toggle() }, h("slot", { name: "title" })), h("section", { onTransitionEnd: () => this.handleTransitionEnd(), class: { "transitioning": this.transitioning }, style: this.style }, h("slot", null))));
  }
  static get is() { return "web-accordion-item"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["web-accordion-item.css"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["web-accordion-item.css"]
    };
  }
  static get properties() {
    return {
      "index": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "index of accordion item from top to bottom"
        },
        "attribute": "index",
        "reflect": true,
        "defaultValue": "-1"
      },
      "open": {
        "type": "boolean",
        "mutable": true,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "accordion item is open or opening (css transition)"
        },
        "attribute": "open",
        "reflect": true,
        "defaultValue": "false"
      },
      "mutationObserverConfig": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "{ childList: boolean; subtree: boolean; }",
          "resolved": "{ childList: boolean; subtree: boolean; }",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The mutation observer config to listen for content changes in the accordion item"
        },
        "defaultValue": "{ childList: true, subtree: true }"
      }
    };
  }
  static get states() {
    return {
      "transitioning": {}
    };
  }
  static get events() {
    return [{
        "method": "openEvent",
        "name": "openEvent",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "triggered when the accordion item is opened"
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }, {
        "method": "contentChanged",
        "name": "contentChanged",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "triggered when the content of the accordion item changes"
        },
        "complexType": {
          "original": "any",
          "resolved": "any",
          "references": {}
        }
      }];
  }
  static get methods() {
    return {
      "closeItem": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "close the accordion item",
          "tags": []
        }
      },
      "openItem": {
        "complexType": {
          "signature": "() => Promise<void>",
          "parameters": [],
          "references": {
            "Promise": {
              "location": "global"
            }
          },
          "return": "Promise<void>"
        },
        "docs": {
          "text": "open the accordion item",
          "tags": []
        }
      }
    };
  }
  static get elementRef() { return "element"; }
  static get watchers() {
    return [{
        "propName": "open",
        "methodName": "stateChanged"
      }];
  }
  static get listeners() {
    return [{
        "name": "contentChanged",
        "method": "recalculateHeight",
        "target": undefined,
        "capture": false,
        "passive": false
      }];
  }
}
