import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';

const webAccordionItemCss = "section{overflow:hidden}.transitioning{-webkit-transition:height .35s ease;transition:height .35s ease}";

const WebAccordionItem$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.openEvent = createEvent(this, "openEvent", 7);
    this.contentChanged = createEvent(this, "contentChanged", 7);
    this.calculatedHeight = 0;
    this.transitioning = false;
    this.index = -1;
    this.open = false;
    this.mutationObserverConfig = { childList: true, subtree: true };
  }
  get style() {
    return {
      height: this.open ? this.calculatedHeight + "px" : "0px"
    };
  }
  stateChanged() {
    this.transitioning = true;
  }
  componentWillLoad() {
    const children = this.element.parentElement.querySelectorAll('web-accordion-item');
    for (let i = 0; i < children.length; i++) {
      const child = children[i];
      if (child == this.element) {
        this.index = i;
      }
    }
    this.mutationObserver = new MutationObserver(() => this.contentChanged.emit());
    this.mutationObserver.observe(this.element, this.mutationObserverConfig);
  }
  componentDidLoad() {
    this.calculateHeight();
  }
  disconnectedCallback() {
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
    }
  }
  /**
   * recalculate Height of accordion item (e.g., when the content of the item changes)
   */
  recalculateHeight() {
    const oldCalculatedHeight = this.calculatedHeight;
    if (this.calculateHeight() != oldCalculatedHeight && this.open) {
      this.transitioning = true;
    }
  }
  /**
   * close the accordion item
   */
  async closeItem() {
    this.open = false;
  }
  /**
   * open the accordion item
   */
  async openItem() {
    this.open = true;
    this.openEvent.emit({
      index: this.index
    });
  }
  toggle() {
    if (this.open) {
      this.closeItem();
    }
    else {
      this.openItem();
    }
  }
  handleTransitionEnd() {
    this.transitioning = false;
  }
  calculateHeight() {
    this.calculatedHeight = this.element.querySelector('web-accordion-item > :not([slot="title"])').clientHeight;
    return this.calculatedHeight;
  }
  render() {
    return (h(Host, null, h("header", { onClick: () => this.toggle() }, h("slot", { name: "title" })), h("section", { onTransitionEnd: () => this.handleTransitionEnd(), class: { "transitioning": this.transitioning }, style: this.style }, h("slot", null))));
  }
  get element() { return this; }
  static get watchers() { return {
    "open": ["stateChanged"]
  }; }
  static get style() { return webAccordionItemCss; }
}, [1, "web-accordion-item", {
    "index": [1538],
    "open": [1540],
    "mutationObserverConfig": [16],
    "transitioning": [32],
    "closeItem": [64],
    "openItem": [64]
  }, [[0, "contentChanged", "recalculateHeight"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["web-accordion-item"];
  components.forEach(tagName => { switch (tagName) {
    case "web-accordion-item":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, WebAccordionItem$1);
      }
      break;
  } });
}

const WebAccordionItem = WebAccordionItem$1;
const defineCustomElement = defineCustomElement$1;

export { WebAccordionItem, defineCustomElement };
