import { r as registerInstance, h, H as Host, g as getElement, c as createEvent } from './index-7a2f74b5.js';

const WebAccordion = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
  }
  openEventHandler(event) {
    const children = this.element.querySelectorAll('web-accordion-item');
    for (let i = 0; i < children.length; i++) {
      if (event.detail.index != i) {
        children[i].closeItem();
      }
    }
  }
  /**
   * Open an accordion item
   * @param index
   */
  async open(index) {
    this.getAccordionItem(index).openItem();
  }
  /**
   * close an accordion item
   * @param index
   */
  async close(index) {
    this.getAccordionItem(index).closeItem();
  }
  getAccordionItem(index) {
    const children = this.element.querySelectorAll('web-accordion-item');
    if (index >= 0 && index < children.length) {
      return children[index];
    }
    else {
      throw new Error("index out of bounds");
    }
  }
  render() {
    return (h(Host, null, h("slot", null)));
  }
  get element() { return getElement(this); }
};

const webAccordionItemCss = "section{overflow:hidden}.transitioning{-webkit-transition:height .35s ease;transition:height .35s ease}";

const WebAccordionItem = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.openEvent = createEvent(this, "openEvent", 7);
    this.contentChanged = createEvent(this, "contentChanged", 7);
    this.calculatedHeight = 0;
    this.transitioning = false;
    this.index = -1;
    this.open = false;
    this.mutationObserverConfig = { childList: true, subtree: true };
  }
  get style() {
    return {
      height: this.open ? this.calculatedHeight + "px" : "0px"
    };
  }
  stateChanged() {
    this.transitioning = true;
  }
  componentWillLoad() {
    const children = this.element.parentElement.querySelectorAll('web-accordion-item');
    for (let i = 0; i < children.length; i++) {
      const child = children[i];
      if (child == this.element) {
        this.index = i;
      }
    }
    this.mutationObserver = new MutationObserver(() => this.contentChanged.emit());
    this.mutationObserver.observe(this.element, this.mutationObserverConfig);
  }
  componentDidLoad() {
    this.calculateHeight();
  }
  disconnectedCallback() {
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
    }
  }
  /**
   * recalculate Height of accordion item (e.g., when the content of the item changes)
   */
  recalculateHeight() {
    const oldCalculatedHeight = this.calculatedHeight;
    if (this.calculateHeight() != oldCalculatedHeight && this.open) {
      this.transitioning = true;
    }
  }
  /**
   * close the accordion item
   */
  async closeItem() {
    this.open = false;
  }
  /**
   * open the accordion item
   */
  async openItem() {
    this.open = true;
    this.openEvent.emit({
      index: this.index
    });
  }
  toggle() {
    if (this.open) {
      this.closeItem();
    }
    else {
      this.openItem();
    }
  }
  handleTransitionEnd() {
    this.transitioning = false;
  }
  calculateHeight() {
    this.calculatedHeight = this.element.querySelector('web-accordion-item > :not([slot="title"])').clientHeight;
    return this.calculatedHeight;
  }
  render() {
    return (h(Host, null, h("header", { onClick: () => this.toggle() }, h("slot", { name: "title" })), h("section", { onTransitionEnd: () => this.handleTransitionEnd(), class: { "transitioning": this.transitioning }, style: this.style }, h("slot", null))));
  }
  get element() { return getElement(this); }
  static get watchers() { return {
    "open": ["stateChanged"]
  }; }
};
WebAccordionItem.style = webAccordionItemCss;

export { WebAccordion as web_accordion, WebAccordionItem as web_accordion_item };
